library(readxl)

#set working directory to the project folder
#setwd("")

localDir = "."
da1 = read.csv(file="data/MERGED pa.csv", header=TRUE, sep = ";")
da2 = read.csv(file="data/MERGED traits.csv", header=TRUE, sep = ";")
da3 = read.csv(file="data/meantemp.csv", header=TRUE, sep = ",")
da3b = read.csv(file="data/snowdepth.csv", header=TRUE, sep = ",")
load(file="data/abup.R") # abundance in previous year

da3$snowdepth = da3b$snowdepth

da2$Non.adult.feeding = as.factor(da2$Non.adult.feeding)
da2$Species = gsub(" ",".",as.character(da2$Species))
sp1 = colnames(da1)[-(1:4)]
sp2 = as.character(da2$Species)
nomatch = which(!(sp1==sp2))
print("not matching species names:")
print(cbind(sp1[nomatch],sp2[nomatch]))

mylm = lm(airTemp~as.factor(Year)+as.factor(Week),data=da3)
pred = predict(mylm,newdata = da3)
da3$airTempImputed = da3$airTemp
mis = is.na(da3$airTemp)
da3$airTempImputed[mis] = pred[mis]
par(mfrow=c(1,2))
plot(da3$airTemp,type="l")
plot(da3$airTempImputed,type="l")

mylm = lm(snowdepth~as.factor(Year)+as.factor(Week),data=da3)
pred = predict(mylm,newdata = da3)
da3$snowdepthImputed = da3$snowdepth
mis = is.na(da3$snowdepth)
da3$snowdepthImputed[mis] = pred[mis]
par(mfrow=c(1,2))
plot(da3$snowdepth,type="l")
plot(da3$snowdepthImputed,type="l")

meabu = colMeans(abup,na.rm = TRUE)
for(i in 1:nrow(abup)){
  mis = is.na(abup[i,])
  abup[i,mis] = meabu[mis]
}
abup = scale(abup)

year = da1$year
years = unique(year)
Y = NULL # the species presence-absence data
Yp = NULL # previous year abundance measured as prevalence
Yabup = NULL # previous year abundance measured as DNA abundance
temp = NULL # temperature of the focal week
clim.summer = NULL # mean temperature of previous summer
clim.winter = NULL # mean snowdepth of previous winter
# summer is defined as snowless and relatively warm period: 10.6.-10.8.
# so winter is from week 33 to 22 and summer from week 23 to week 32.
sd.year = NULL # study-design: year
sd.week = NULL # study-design: week
sd.trap = NULL # study-design: trap
for(fyear in years){
  da1c = subset(da1,year==fyear)
  yc = da1c[,-(1:4)]
  da1p = subset(da1,year==(fyear-1))
  yp = da1p[,-(1:4)]
  da3c = subset(da3,Year==fyear)
  da3p = subset(da3,Year==(fyear-1))
  if(dim(yp)[1]>0){
    n = dim(yc)[1]
    for(i in 1:n){
      fweek = da1c$week[i]
#      selrows = which(da3c$Week==%in% (fweek-(0:3)))
      selrows = which(da3c$Week==fweek)
      if(length(selrows)==1){
        Yp = rbind(Yp,colMeans(yp))
        Yabup = rbind(Yabup,abup[rownames(abup)==(fyear-1),])
        Y = rbind(Y,yc[i,])
        temp = c(temp,mean(da3c[selrows,]$airTempImputed))
        clim.summer = c(clim.summer,mean(da3p[da3p$Week %in% 23:32,]$airTempImputed))
        snowts = c(da3p[da3p$Week %in% 33:52,]$snowdepthImputed,
                 da3c[da3c$Week %in% 1:32,]$snowdepthImputed)
        clim.winter = c(clim.winter,mean(snowts))
        sd.year = c(sd.year,fyear)
        sd.week = c(sd.week,fweek)
        sd.trap = c(sd.trap,as.character(da1c$trap[i]))
      }
    }
  }
}
Yp = as.matrix(Yp)
Yabup = as.matrix(Yabup)

Y = as.matrix(Y)
plot(rowSums(Y),type="l")
plot(temp,type="l")
studyDesign = data.frame(year = as.factor(sd.year), 
                         week = as.factor(sd.week), 
                         trap = as.factor(sd.trap))

ny = dim(Y)[1]
ns = dim(Y)[2]
group = da2$Non.adult.feeding
grouptable = table(group)
groups = names(grouptable[grouptable>5])
ng = length(groups)
spp = colnames(Y)
Xz = matrix(nrow=ny,ncol=4+ns)
colnames(Xz)=c("temperature","clim.summer","clim.winter","week",spp)
Xz[,1] = temp
Xz[,2] = clim.summer
Xz[,3] = clim.winter
Xz[,4] = sd.week
Xz[,4+(1:ns)] = Yp[,]
Xz = data.frame(Xz)

Xzabu = Xz
Xzabu[,4+(1:ns)] = Yabup[,]

X = matrix(nrow=ny,ncol=5+ng)
colnames(X)=c("temperature","clim.summer","clim.winter","week","within.species",groups)
X[,1] = temp
X[,2] = clim.summer
X[,3] = clim.winter
X[,4] = sd.week
for(i in 1:ng){
  X[,5+i] = rowMeans(Yp[,group == groups[i]])
}
X = data.frame(X)

Xabu = X
for(i in 1:ng){
  Xabu[,5+i] = rowMeans(Yabup[,group == groups[i]])
}

XData = list()
for(k in 1:ns){
  fX = X
  fX[,5] = Yp[,k]
  XData[[k]] = fX
}

XData.abu = list()
for(k in 1:ns){
  fX = Xabu
  fX[,5] = Yabup[,k]
  XData.abu[[k]] = fX
}

TrData = data.frame(group = group)

save(file = "data/hmscData.RData",XData,XData.abu,Xz, Xzabu,Y,TrData,studyDesign)
