library(Hmsc)
set.seed(1)
#set working directory to the project folder
#setwd("")

localDir = "."
data.directory = file.path(localDir, "data")
model.directory = file.path(localDir, "models")
model.fit.directory = file.path(localDir, "model fits")

nChains = 2
samples = 1000
thin = 10
simodel = 7 #which version of species interactions model to use
#prior probability is 0.01 (4), 0.1 (5), 0.5 (6) or 1.0 (7)
me = matrix(NA,nrow=28,ncol=7)
colnames(me) = c("Model","Abundance measure","Climatic predictor","corr","corr-CV","AUC","AUC-CV")
index = 0
for(climtype in 1:2){
  for(abutype in c(1,3)){
    res = list()
    for(i in 1:7){
      index = index + 1
      filename=file.path(model.directory, paste0("model_",as.character(i),
                                                 "_climtype_",as.character(climtype),
                                                 "_abutype_",as.character(abutype),
                                                 "_chains_",as.character(nChains),
                                                 "_samples_",as.character(samples),
                                                 "_thin_",as.character(thin)))
      load(filename)
      
      filename=file.path(model.fit.directory, paste0("mf_model_",as.character(i),
                                                     "_climtype_",as.character(climtype),
                                                     "_abutype_",as.character(abutype),
                                                     "_chains_",as.character(nChains),
                                                     "_samples_",as.character(samples),
                                                     "_thin_",as.character(thin)))
      load(file=filename) # partition,MF,MFCV,mpred,mpredCV
      
      year = m$studyDesign$year
      years = levels(year)
      ny = length(years)
      yData = matrix(NA,nrow = ny, ncol = m$ns)
      yPred = matrix(NA,nrow = ny, ncol = m$ns)
      yPredCV = matrix(NA,nrow = ny, ncol = m$ns)
      for(j in 1:ny){
        yData[j,] = colMeans(m$Y[year == years[j],])
        yPred[j,] = colMeans(mpred[year == years[j],])
        yPredCV[j,] = colMeans(mpredCV[year == years[j],])
      }
      #image(yData)
      #image(yPred)
      #image(yPredCV)
      correlation = rep(0,m$ns)
      correlationCV = rep(0,m$ns)
      yRMSE = rep(0,m$ns)
      yRMSECV = rep(0,m$ns)
      for(j in 1:m$ns){
        correlation[j] = cor(yData[,j],yPred[,j])
        correlationCV[j] = cor(yData[,j],yPredCV[,j])
        yRMSE[j] = sqrt(mean((yData[,j]-yPred[,j])^2))
        yRMSECV[j] = sqrt(mean((yData[,j]-yPredCV[,j])^2))  }
      MF$corr = correlation
      MFCV$corr = correlationCV
      me[index,] = c(i,abutype,climtype,mean(correlation,na.rm = TRUE),mean(correlationCV,na.rm = TRUE),mean(MF$AUC),mean(MFCV$AUC))
      MF$yRMSE = yRMSE
      MFCV$yRMSE = yRMSECV
      res[[i]] = list(MF=MF,MFCV=MFCV, WAIC=WAIC)
    }
    
    c(res[[1]]$WAIC,res[[2]]$WAIC,res[[3]]$WAIC,res[[simodel]]$WAIC)
    
    models = c("Abiotic model","Density-dependent model",
               "Trophic-interactions model","Species interactions 0.01","Species interactions 0.1",
               "Species interactions 0.5", "Species-interactions model")
    mycols = c("black","blue","cyan","red","green")
    
    pdf(paste0("results/model_fits_abutype_",as.character(abutype),
               "_climtype_",as.character(climtype),".pdf"),width = 8,height = 5)
    par(mfrow=c(2,3))
    for(i in c(1,2,simodel-1)){
      ave = round(100*mean(res[[i+1]]$MF$corr-res[[1]]$MF$corr))
      impr = round(100*mean(res[[i+1]]$MF$corr>res[[1]]$MF$corr))
      plot(res[[1]]$MF$corr,res[[i+1]]$MF$corr,xlim=c(-1,1),ylim=c(-1,1),
           col=mycols[m$TrData$group],pch=16,
           xlab = paste0("Explanatory power (",models[[1]],")"),
           ylab = paste0("Explanatory power (",models[[i+1]],")"),
           main = paste0("Average improvement = ",as.character(ave),"%\n",
                         "Species with improved power = ",as.character(impr),"%\n"),
           cex.main=0.9)
      mtext(c("(a)","(b)","(c)")[i], side = 3, adj = -0.2, line = 0.8)
      abline(0,1)
      abline(h = 0,col="gray")
      abline(v = 0,col="gray")
      groups = levels(m$TrData$group)
      groups[groups=="Omnivorous"]="Omnivore"
      legend(0.05,-0.3, groups, pch = 16, pt.bg = "white", col = mycols, cex=0.6)
    }
    for(i in c(1,2,simodel-1)){
      ave = round(100*mean(res[[i+1]]$MFCV$corr-res[[1]]$MFCV$corr))
      impr = round(100*mean(res[[i+1]]$MFCV$corr>res[[1]]$MFCV$corr))
      plot(res[[1]]$MFCV$corr,res[[i+1]]$MFCV$corr,xlim=c(-1,1),ylim=c(-1,1),
           col=mycols[m$TrData$group],pch=16,
           xlab = paste0("Predictive power (",models[[1]],")"),
           ylab = paste0("Predictive power (",models[[i+1]],")"),
           main = paste0("Average improvement = ",as.character(ave),"%\n",
                         "Species with improved power = ",as.character(impr),"%\n"),
           cex.main=0.9)
      mtext(c("(d)","(e)","(f)")[i], side = 3, adj = -0.2, line = 0.8)
      abline(0,1)
      abline(h = 0,col="gray")
      abline(v = 0,col="gray")
      groups = levels(m$TrData$group)
      groups[groups=="Omnivorous"]="Omnivore"
      legend(0.05,-0.3, groups, pch = 16, pt.bg = "white", col = mycols, cex=0.6)
    }
    dev.off()
    
    #pdf("results/model_fits_prevalence.pdf",width = 8,height = 8)
    tiff(paste0("results/model_fits_prevalence_abutype_",as.character(abutype),
                "_climtype_",as.character(climtype),
                ".tiff"),width = 20,height = 20,units = "cm",res = 300)
    par(mfrow=c(2,2))
    gr = m$TrData$group
    sel=!gr=="Omnivorous"
    gr = gr[sel]
    grs = unique(as.character(gr))
    for(i in c(1,2,3,simodel)){
      prev=log(colSums(m$Y),base=10)
      mi = min(prev)
      ma = max(prev)
      pres = c(mi,ma)
      yy = res[[i]]$MFCV$corr
      plot(prev,yy,col=mycols[m$TrData$group],pch=16,ylim=c(-1,1),
           xlab = "log_10(number of occurrences)",
           ylab = paste0("Predictive power"),
           main = models[[i]])
      mtext(c("(a)","(b)","(c)","(d)")[i], side = 3, adj = -0.2, line = 0.5)
      prev = prev[sel]
      yy = yy[sel]
      m1 = lm(yy~1)
      m2 = lm(yy~prev)
      m3 = lm(yy~gr)
      m4 = lm(yy~prev+gr)
      a=AIC(m1,m2,m3,m4)
      bm = which(a$AIC==min(a$AIC))
      mm = list(m1,m2,m3,m4)[[bm]]
      for(j in 1:length(grs)){
        g = grs[j]
        lines(pres,predict(mm,newdata = data.frame(gr=rep(g,2),prev=pres)),
              col=if(bm>2){mycols[which(levels(m$TrData$group)==g)]}else{"gray"})
      }
    }
    dev.off()
    
    co.exp = matrix(NA,4,5)
    co.pred = matrix(NA,4,5)
    group = m$TrData$group
    for(j in 1:4){
      jj = j
      if(j==4){jj = simodel}
      for(i in 1:4){
        co.exp[j,i] = mean(res[[jj]]$MF$corr[group==grs[[i]]])
        co.pred[j,i] = mean(res[[jj]]$MFCV$corr[group==grs[[i]]])
      }
      co.exp[j,5]= mean(res[[jj]]$MF$corr)
      co.pred[j,5]= mean(res[[jj]]$MFCV$corr)
    }
    round(co.exp,2)
    round(co.pred,2)
    grs
    
    c(mean(res[[2]]$MF$corr>res[[1]]$MF$corr),mean(res[[3]]$MF$corr>res[[1]]$MF$corr),mean(res[[simodel]]$MF$corr>res[[1]]$MF$corr))
    c(mean(res[[2]]$MFCV$corr>res[[1]]$MFCV$corr),mean(res[[3]]$MFCV$corr>res[[1]]$MFCV$corr),mean(res[[simodel]]$MFCV$corr>res[[1]]$MFCV$corr))
  }
}

sel = c(1,15,2,16,9,23,3,17,10,24,4,18,11,25,5,19,12,26,6,20,13,27,7,21,14,28)
mesel = me[sel,]
for(i in 4:7){
  mesel[,i]=round(mesel[,i],3)
}
mesel[mesel[,1]==1,1] ="Abiotic"
mesel[mesel[,1]==2,1] ="Density-dependent"
mesel[mesel[,1]==3,1] ="Trophic-interactions"
mesel[mesel[,1]==4,1] ="Species-interactions"
mesel[mesel[,1]==5,1] ="Species-interactions_0.1"
mesel[mesel[,1]==6,1] ="Species-interactions_0.5"
mesel[mesel[,1]==7,1] ="Species-interactions_1.0"
mesel[mesel[,2]==1,2] ="Prevalence"
mesel[mesel[,2]==3,2] ="DNA-abundance"
mesel[mesel[,3]==1,3] ="Summer temperature"
mesel[mesel[,3]==2,3] ="Snow depth"
write.csv(mesel,file = "results/model_comparison.csv")
