library(Hmsc)
set.seed(1)
#set working directory to the project folder
#setwd("")

localDir = "."
data.directory = file.path(localDir, "data")
model.directory = file.path(localDir, "models")

nChains = 2
samples = 1000
thin = 100
VPmeans = matrix(NA,nrow=8,ncol=8)
colnames(VPmeans)=c("Week temperature","Summer/Winter","Week",
                    "Density dependence","Trophic interactions","Species interactions","Trap","Year")
for(abutype in 3){
  for(climtype in c(1:2)){
    for (i in c(1,2,3,7)){
      filename=file.path(model.directory, paste0("model_",as.character(i),
                                                 "_climtype_",as.character(climtype),
                                                 "_abutype_",as.character(abutype),
                                                 "_chains_",as.character(nChains),
                                                 "_samples_",as.character(samples),
                                                 "_thin_",as.character(thin)))
      load(filename)
      if(i==1){
        groupnames = c("temperature",m$covNames[3],"week")
        group = c(1,1,2,3,3)
      }
      if(i==2){
        groupnames = c("temperature",m$covNames[3],"week","within species")
        group = c(1,1,2,3,3,4)
      }
      if(i==3){
        groupnames = c("temperature",m$covNames[3],"week","within species","among groups")
        group = c(1,1,2,3,3,4,5,5,5,5)
      }
      if(i>3){
        groupnames = c("temperature",m$covNames[3],"week","among species")
        group = rep(4,m$nc)
        group[1]=1
        group[2]=1
        group[3]=2
        group[4]=3
        group[5]=3
      }
      tiff(paste0("results/Fig2_model_",as.character(i),
                  "_abutype_",as.character(abutype),
                  "_climtype_",as.character(climtype),
                  ".tiff"),width = 20,height = 10,units = "cm",res = 300)
      VP = computeVariancePartitioning(m, group = group, groupnames = groupnames)
      tmp = rowMeans(VP$vals)
      if(i==1 && climtype==1){
        VPmeans[1,] = c(tmp[1:3],0,0,0,tmp[4:5])
      }
      if(i==1 && climtype==2){
        VPmeans[2,] = c(tmp[1:3],0,0,0,tmp[4:5])
      }
      if(i==2 && climtype==1){
        VPmeans[3,] = c(tmp[1:4],0,0,tmp[5:6])
      }
      if(i==2 && climtype==2){
        VPmeans[4,] = c(tmp[1:4],0,0,tmp[5:6])
      }
      if(i==3 && climtype==1){
        VPmeans[5,] = c(tmp[1:5],0,tmp[6:7])
      }
      if(i==3 && climtype==2){
        VPmeans[6,] = c(tmp[1:5],0,tmp[6:7])
      }
      if(i==7 && climtype==1){
        VPmeans[7,] = c(tmp[1:3],0,0,tmp[4:6])
      }
      if(i==7 && climtype==2){
        VPmeans[8,] = c(tmp[1:3],0,0,tmp[4:6])
      }
      plotVariancePartitioning(m,VP)
      dev.off()
      
      tiff(paste0("results/Fig3_model_",as.character(i),
                  "_abutype_",as.character(abutype),
                  "_climtype_",as.character(climtype),
                  ".tiff"),width = 20,height = 10,units = "cm",res = 300)
      postBeta = getPostEstimate(m, parName="Beta")
      if(i==4){
        tmp = postBeta$support
        for(j1 in 1:m$ns){
          for(j2 in 1:m$ns){
            if(postBeta$support[5+j1,j2]<0.5 & postBeta$supportNeg[5+j1,j2]<0.5){
              tmp[5+j1,j2] = 0.5
            }
          }
        }
        postBeta$support = tmp
        tmp = tmp[6:(5+m$ns),]
        zzn = which(tmp<0.1,arr.ind = TRUE)
        zzp = which(tmp>0.9,arr.ind = TRUE)
        nneg = dim(zzn)[1]
        npos = dim(zzp)[1]
        print(nneg,npos)
        #sptosp = data.frame(direction = c(rep("Negative",nneg),rep("Positive",npos)))
        #sptosp$influencer = as.character(rep("x",nneg+npos))
        #sptosp$focal = as.character(rep("x",nneg+npos))
        #for(j in 1:nneg){
        #  sptosp[j,]$influencer = m$spNames[zzn[j,1]]
        #  sptosp[j,]$focal = m$spNames[zzn[j,2]]
        #}
        #write.csv(sptosp,file="results/sptosp.csv")
      }
      ord=order(m$TrData$group)
      tr = m$TrData$group[ord]
      #m$TrData$group[ord]
      postBeta$mean=postBeta$mean[,ord]
      postBeta$support=postBeta$support[,ord]
      if(i==3){
        groups = c("Herbivore","Parasitoid","Predator","Saprophage_Detritivore")
        jpreds = c(2,3,6,7,8,9,10)
        pota = matrix(NA,nrow = length(jpreds),ncol = 4)
        neta = matrix(NA,nrow = length(jpreds),ncol = 4)
        for(jpred in 1:length(jpreds)){
          for(jresp in 1:4){
            tmp = postBeta$support[jpreds[jpred],tr==groups[jresp]]
            pota[jpred,jresp] = mean(tmp>0.9)
            neta[jpred,jresp] = mean(tmp<0.1)      }
        }
        colnames(pota) = groups
        rownames(pota) = m$covNames[jpreds]
        colnames(neta) = groups
        rownames(neta) = m$covNames[jpreds]
        print("pota = ")
        print(pota)
        print("neta = ")
        print(neta)
        if(climtype==1){
          pota1 = pota
          neta1 = neta
        }
        if(climtype==2){
          pota2 = pota
          neta2 = neta
        }
      }
      #cbind(1:52,as.character(m$TrData$group[ord]),postBeta$support[6,])
      plotBeta(m, post=postBeta, param = "Sign",spNamesNumbers=c(F, FALSE),
               supportLevel = 0.9)
      dev.off()
      
      tiff(paste0("results/Fig4_model_",as.character(i),
                  "_abutype_",as.character(abutype),
                  "_climtype_",as.character(climtype),
                  ".tiff"),width = 20,height = 10,units = "cm",res = 300)
      postGamma = getPostEstimate(m, parName="Gamma")
      plotGamma(m, post=postGamma, supportLevel = 0.8)
      dev.off()
      
      if(i==3){
        groups = c("Herbivore","Parasitoid","Predator","Saprophage_Detritivore")
        res = matrix(0,nrow = 4,ncol = 4)
        for(j1 in 1:4){
          if(climtype==1){
            Gradient = constructGradient(m,focalVariable = groups[j1],
                                         non.focalVariables = list("week"=list(1),
                                                                   "within.species.pa"=list(1),
                                                                   "temperature"=list(1),
                                                                   "clim.summer"=list(1)))
          } else {
            Gradient = constructGradient(m,focalVariable = groups[j1],
                                         non.focalVariables = list("week"=list(1),
                                                                   "within.species.pa"=list(1),
                                                                   "temperature"=list(1),
                                                                   "clim.winter"=list(1)))
          }
          predY = predict(m, Gradient=Gradient, expected = TRUE)  
          plotGradient(m, Gradient, pred=predY, measure="Y",index=ord[29], 
                       showData = TRUE)
          tiff(paste0("results/pred_",groups[j1],
                      "_abutype_",as.character(abutype),
                      "_climtype_",as.character(climtype),
                      ".tiff"),width = 18,height = 18,units = "cm",res = 300)
          par(mfrow=c(2,2))
          for(j2 in 1:4){
            sel = m$TrData$group==groups[j2]
            predY2=predY
            for(k in 1:length(predY)){
              predY2[[k]] = predY[[k]][,sel]
            }
            m2 = m
            m2$Y = m$Y[,sel]    
            res[j1,j2]=plotGradient(m2, Gradient, pred=predY2, measure="S", 
                                    showData = TRUE, ylabel = groups[j2])
          }
          dev.off()
        }
        res
      }
    }
  }
}

if(abutype==1){
  pdf("results/figure_3.pdf",width = 8,height = 8)
} else {
  pdf("results/figure_3_DNA_abu.pdf",width = 8,height = 8)  
}
par(mfrow=c(3,1),mar=c(3.7,4.1,4.1,0.1))
tmp = rainbow(8)
tmp[1] = "firebrick1"
tmp[2] = "darkorange"
tmp[3] = "seagreen3"
tmp[4] = "deepskyblue"
tmp[5] = "deepskyblue3"
tmp[6] = "deepskyblue4"
tmp[7] = "gray50"
tmp[8] = "gray"
barplot(cbind(t(VPmeans)),xlim=c(0.5,12.5),
        ylab="Proportion of variance",cex.lab=1.2,cex.axis = 1.2,
        col=tmp,legend.text = TRUE,args.legend = c(cex=1.2),main="(a)",cex.main=2,adj=0)
text(x=0.7+1.2*(0:7), y=-0.1,adj=0.5,
     labels = c("Summer","Winter","Summer","Winter","Summer","Winter","Summer","Winter"), srt = 0, xpd = TRUE,cex=1.2)
text(x=-0.3, y=-0.1,adj=0.5,
     labels = c("Variant:"), srt = 0, xpd = TRUE,cex=1.2,font = 2)
text(x=1.3+2.4*(0:3), y=-0.25,adj=0.5,
     labels = c("Abiotic","Density-dependent",
                "Trophic-interactions","Species-interactions"),
     srt = 0, xpd = TRUE,cex=1.2)
text(x=-0.3, y=-0.25,adj=0.5,
     labels = c("Model:"), srt = 0, xpd = TRUE,cex=1.2,font = 2)
for(zz in 1:2){
  if(zz==1){
    pota = pota1
    neta = neta1
  }
  if(zz==2){
    pota = pota2
    neta = neta2
  }
  ta = c()
  na = c()
  labs = rownames(pota)
  labs = c("Week\n temperature","Summer\n temperature","Density\n dependence",
           "Herbivore","Parasitoid","Predator","Saprophage-\n Detritivore")
  if(zz==2){labs = c("Week temperature","Winter\n snow depth","Density\n dependence",
                     "Herbivore","Parasitoid","Predator","Saprophage-\n Detritivore")}
  for(z in 1:7){
    ta = cbind(ta,t(cbind(pota[z,],neta[z,])))
    na = c(na,"He","Pa","Pr","SD")
    if(z<7){
      ta = cbind(ta,c(0,0))
      na = c(na,"")
    }
  }
  par(xaxt="n")
  barplot(ta,col=c("red","blue"),ylim=c(0,1), 
          ylab="Proportion of species",cex.lab=1.2,cex.axis = 1.2, cex.names  = 2,
          main=if(zz==1){("(b)")}else{"(c)"},cex.main=2,adj=0)
  abline(h=0)
  text(x=2.5+6*(0:6), y=-0.2,adj=0.5,
       labels = labs, srt = 0, xpd = TRUE,cex=1.2)
  text(x=0.7+1.2*(0:33),y=0.01,labels=na,pos = 1, xpd = TRUE,cex=1.2)
}
dev.off()

